package edu.uoc.www.petercrab.Activities;

import android.content.Context;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.widget.Toast;

import java.util.Locale;

import edu.uoc.www.petercrab.Utils.SharedPreferencesUtil;

/**
 * Esta actividad hace de base del resto y contiene la funcionalidad que tienen en común todas
 * las activities del proyecto
 */
public class BaseActivity extends AppCompatActivity {

    /**
     * Lenguaje actual de la aplicación
     */
    protected String currentLanguage = null;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRequestedOrientation (ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        defaultLanguage();
    }

    /**
     * Va a otra activity, cerrando la activity de procedencia.
     * @param context
     * @param cls
     */
    public void goToActivity (Context context, Class<?> cls) {

        Intent i = new Intent(context, cls);
        finish();
        startActivity(i);

    }

    /**
     * Abre la activity principal, cerrando la anterior
     * @param context
     */
    public void goToMain (Context context) {

        goToActivity(context,MainActivity.class);

    }

    /**
     * Setea el lenguaje de la aplicación al pasado por parámetro
     * @param lang
     */
    public void setLanguage (String lang) {

        String languageToLoad  = lang; // your language
        Locale locale = new Locale(languageToLoad);
        Locale.setDefault(locale);
        Configuration config = new Configuration();
        config.locale = locale;
        getBaseContext().getResources().updateConfiguration(config,getBaseContext().getResources().getDisplayMetrics());

    }

    /**
     * Setea el lenguaje pasado como el por defecto de la aplicación
     * y va a la activity main
     * @param lang
     */
    public void setDefaultLanguage (String lang) {

        new SharedPreferencesUtil(this).setDefaultLanguage(lang);

        defaultLanguage();
        goToMain(getBaseContext());

    }

    /**
     * Setea el lenguaje actual mirando primero si se seteo en el menú
     * y de no ser así coge el que tiene configurado el móvil
     * Actualmente solo se dispone de francés, inglés, catalán y español.
     */
    public void defaultLanguage () {

        String defaultLanguage =  new SharedPreferencesUtil(this).getDefaultLanguage();

        if (defaultLanguage!=null) {
            currentLanguage = defaultLanguage;
        }
        else {
            currentLanguage = getResources().getConfiguration().locale.getLanguage();
        }

        //Only available in these languages:
        if (!currentLanguage.contentEquals("fr")
                && !currentLanguage.contentEquals("en")
                && !currentLanguage.contentEquals("es")
                && !currentLanguage.contentEquals("ca")
                ) {
            currentLanguage = "en";
        }


        setLanguage(currentLanguage);

    }

    /**
     * Muestra un mensaje toast.
     * @param context
     * @param message
     */
    protected void showMessage (Context context, String message) {

        Toast.makeText(context,message,
                Toast.LENGTH_LONG).show();

    }

}
