package edu.uoc.www.petercrab.Activities;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.design.widget.NavigationView;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.Spinner;
import android.widget.TextView;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import edu.uoc.www.petercrab.Fragments.LanguageDialogFragment;
import edu.uoc.www.petercrab.Fragments.MapFragment;
import edu.uoc.www.petercrab.Fragments.RankingFragment;
import edu.uoc.www.petercrab.Fragments.ScoreFragment;
import edu.uoc.www.petercrab.Fragments.UpdateFragment;
import edu.uoc.www.petercrab.Models.City;
import edu.uoc.www.petercrab.Models.UserInfo;
import edu.uoc.www.petercrab.R;

public class MainActivity extends BaseActivity
        implements NavigationView.OnNavigationItemSelectedListener,
                    MapFragment.OnFragmentInteractionListener,
                    ScoreFragment.OnFragmentInteractionListener,
                    UpdateFragment.OnFragmentInteractionListener,
                    RankingFragment.OnListFragmentInteractionListener{

    private NavigationView navigationView;
    private FirebaseAuth mAuth;
    private FirebaseUser currentUser;

    private UserInfo userInfo;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_main);


        mAuth = FirebaseAuth.getInstance();

        currentUser = mAuth.getCurrentUser();

        if (currentUser == null) {

            goToActivity(MainActivity.this,RegisterActivity.class);

        }
        else {

            loadUserInfo(currentUser.getUid());

        }

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.addDrawerListener(toggle);
        toggle.syncState();

        navigationView = (NavigationView) findViewById(R.id.nav_view);
        navigationView.setNavigationItemSelectedListener(this);

        View hView =  navigationView.getHeaderView(0);

        TextView name  =  hView.findViewById(R.id.nav_header_title);
        TextView email =  hView.findViewById(R.id.nav_header_subtitle);

        String[] items = new String[]{"Barcelona"};

        Spinner spinner = (Spinner) navigationView.getMenu().findItem(R.id.navigation_drawer_item3).getActionView();
        spinner.setAdapter(new ArrayAdapter<String>(this,android.R.layout.simple_spinner_dropdown_item,items));
        spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                //Toast.makeText(MainActivity.this,items[position],Toast.LENGTH_SHORT).show();
            }
            @Override
            public void onNothingSelected(AdapterView<?> parent) {
            }
        });


        name.setText(currentUser.getDisplayName());
        email.setText(currentUser.getEmail());

        Uri photoURL = currentUser.getPhotoUrl();

        if (photoURL != null) {

            ImageView userPic = hView.findViewById(R.id.user_pic);

            Picasso.get().load(photoURL).into(userPic);

        }

    }

    /**
     * Carga la información relativa al usuario.
     * @param uid
     */
    public void loadUserInfo (String uid) {

        FirebaseDatabase database = FirebaseDatabase.getInstance();

        final DatabaseReference dbRef = database.getReference("users");

        final ValueEventListener listener = new ValueEventListener() {

            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                GenericTypeIndicator<UserInfo> genericTypeIndicator =new GenericTypeIndicator<UserInfo>(){};
                userInfo = dataSnapshot.getValue(genericTypeIndicator);

                //Marca en el menú el  mapa como elemento seleccionado por defecto
                navigationView.getMenu().getItem(1).setChecked(true);
                loadFragment(MapFragment.newInstance(userInfo));

            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.w("PETER_CRAB", "Failed to read value.", error.toException());
            }
        };

        dbRef.child(uid).addListenerForSingleValueEvent(listener);


    }

    @Override
    public void onBackPressed() {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        if (drawer.isDrawerOpen(GravityCompat.START)) {
            drawer.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {

            FragmentManager fragmentManager = getSupportFragmentManager();
            LanguageDialogFragment dialog = new LanguageDialogFragment();
            dialog.show(fragmentManager, "tagAlerta");

            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    /**
     * Dependiendo de la opción elegida, carga un fragment u otro.
     * @param item
     * @return
     */
    @SuppressWarnings("StatementWithEmptyBody")
    @Override
    public boolean onNavigationItemSelected(MenuItem item) {
        // Handle navigation view item clicks here.
        int id = item.getItemId();

        if (id == R.id.nav_map) {
            loadFragment(MapFragment.newInstance(userInfo));
        }
        else if (id == R.id.nav_ranking) {
            loadFragment(RankingFragment.newInstance(1));
        }
        else if (id == R.id.nav_score) {
            loadFragment(ScoreFragment.newInstance(userInfo));
        }
        else if (id == R.id.nav_update) {
            loadFragment(UpdateFragment.newInstance(userInfo));
        }
        else if (id == R.id.nav_logout) {

            FirebaseAuth.getInstance().signOut();
            City.points = null;
            City.lastUpdate = null;
            finish();
            startActivity(new Intent(MainActivity.this,LoginActivity.class));

        }

        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        drawer.closeDrawer(GravityCompat.START);
        return true;
    }

    private void loadFragment (Fragment fragment) {

        FragmentManager manager = getSupportFragmentManager();
        manager.beginTransaction().replace(R.id.main_container,fragment).commit();

    }

    @Override
    public void onFragmentInteraction(Uri uri) {

    }

    @Override
    public void onListFragmentInteraction(UserInfo item) {

    }
}
