package edu.uoc.www.petercrab.Activities;

import android.content.Intent;
import android.support.annotation.NonNull;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;

import com.google.firebase.auth.UserProfileChangeRequest;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;

import java.util.HashMap;
import java.util.Map;

import edu.uoc.www.petercrab.Models.UserInfo;
import edu.uoc.www.petercrab.R;

public class RegisterActivity extends BaseActivity {

    private ProgressBar progressBar;
    private Button registerButton;
    private EditText emailInput, passwordInput, repeatedPasswordInput,nameInput;
    private FirebaseAuth mAuth;
    private FirebaseUser user;
    private TextView loginLink;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_register);

        mAuth = FirebaseAuth.getInstance();
        user = mAuth.getCurrentUser();

        if (user != null) {

            updateUI(user);

        }

        progressBar = findViewById(R.id.progressBar);

        registerButton = findViewById(R.id.registerButton);

        nameInput             = findViewById(R.id.nameInput);
        emailInput            = findViewById(R.id.emailInput);
        passwordInput         = findViewById(R.id.passwordInput);
        repeatedPasswordInput = findViewById(R.id.passwordRepeatInput);
        loginLink             = findViewById(R.id.loginLink);

        registerButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                String name            = nameInput.getText().toString().trim();
                String email           = emailInput.getText().toString().trim();
                String password        = passwordInput.getText().toString().trim();
                String repeatedPasword = repeatedPasswordInput.getText().toString().trim();


                register (name,email,password,repeatedPasword);

            }
        });

        loginLink.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                goToActivity(RegisterActivity.this,LoginActivity.class);

            }
        });


    }

    protected void updateUI (FirebaseUser user) {

        startActivity(new Intent(RegisterActivity.this,MainActivity.class));

    }


    protected boolean checkPassword (String password) {

        return !password.isEmpty();

    }

    protected void register (final String name, String email, String password, String repeated_password) {

        if (    TextUtils.isEmpty(name) ||
                TextUtils.isEmpty(email) ||
                TextUtils.isEmpty(password) ||
                TextUtils.isEmpty(repeated_password)
                ) {

            showMessage(this,getString(R.string.error_empty_fields));
            return;

        }
        if (!checkPassword(password) || !checkPassword(repeated_password)) {
            showMessage(this,getString(R.string.error_pass_format));
            return;
        }
        if (!password.contentEquals(repeated_password)) {
            showMessage(this,getString(R.string.password_not_match));
            return;
        }

        progressBar.setVisibility(View.VISIBLE);

        /**
         * Crea el usuario a partir del email y la contraseña
         */
        mAuth.createUserWithEmailAndPassword(email,password)
                .addOnCompleteListener(this, new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {
                        if (task.isSuccessful()) {
                            user = mAuth.getCurrentUser();

                            if (user != null) {

                                /**
                                 * Una vez creado, se actualiza su perfil con el nombre que se muestra.
                                 */
                                UserProfileChangeRequest profileUpdates = new UserProfileChangeRequest.Builder()
                                        .setDisplayName(name)
                                        .build();

                                user.updateProfile(profileUpdates)
                                        .addOnCompleteListener(new OnCompleteListener<Void>() {
                                            @Override
                                            public void onComplete(@NonNull Task<Void> task) {
                                                if (task.isSuccessful()) {

                                                    FirebaseDatabase database = FirebaseDatabase.getInstance();

                                                    mAuth = FirebaseAuth.getInstance();

                                                    FirebaseUser currentUser = mAuth.getCurrentUser();

                                                    /**
                                                     * Se crea el usuario en la base de datos Firebase Database
                                                     * con la información necesaria para el ranking y guardar
                                                     * los puntos visitados.
                                                     */
                                                    DatabaseReference dbRef = database.getReference("users");
                                                    UserInfo userInfo = new UserInfo();
                                                    userInfo.setNickname(name);
                                                    userInfo.setScore(0);
                                                    userInfo.setOrder(0);

                                                    Map<String,Object> fields = new HashMap<String,Object>();

                                                    fields.put(currentUser.getUid(), userInfo );
                                                    dbRef.updateChildren(fields);

                                                    updateUI(user);
                                                }
                                            }
                                        });
                            }

                        } else {

                            showMessage (getApplicationContext(),"" + getText(R.string.error_registry));

                        }

                        progressBar.setVisibility(View.GONE);
                    }
                });
    }

    @Override
    protected void onResume() {
        super.onResume();
        progressBar.setVisibility(View.GONE);
    }

}
