package edu.uoc.www.petercrab.Fragments;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapView;
import com.google.android.gms.maps.MapsInitializer;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import edu.uoc.www.petercrab.Activities.DetailActivity;
import edu.uoc.www.petercrab.Activities.MainActivity;
import edu.uoc.www.petercrab.Models.City;
import edu.uoc.www.petercrab.Models.Point;
import edu.uoc.www.petercrab.Models.UserInfo;
import edu.uoc.www.petercrab.R;
import edu.uoc.www.petercrab.Utils.SharedPreferencesUtil;

/**
 * Fragmento que muestra el mapa con los duendes.
 */
public class MapFragment extends Fragment implements OnMapReadyCallback {

    private MapView mapView;
    private GoogleMap mGoogleMap;
    private View rootView;

    private UserInfo userInfo;
    private MainActivity mActivity;


    private LocationManager locationManager;
    private LocationListener locationListener;
    private Marker personMarker;


    private static final int PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION = 1001 ;
    private static final float APPROPRIATE_ZOOM_LEVEL_MAP = 17.0f;
    private static final int POINTS_PER_ELF = 10;
    private static final int POINT_REACHED_DISTANCE = 20;


    private boolean mLocationPermissionGranted;
    private FirebaseAuth mAuth;
    private FirebaseUser currentUser;

    private ProgressDialog progress;

    private static final String ARG_PARAM1 = "userInfo";

    private OnFragmentInteractionListener mListener;

    public MapFragment() {
        // Required empty public constructor
    }


    public static MapFragment newInstance(UserInfo userInfo) {
        MapFragment fragment = new MapFragment();
        Bundle args = new Bundle();
        args.putSerializable(ARG_PARAM1,userInfo);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            userInfo = (UserInfo) getArguments().getSerializable(ARG_PARAM1);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        rootView = inflater.inflate(R.layout.fragment_map, container, false);


        return rootView;
    }


    public void onButtonPressed(Uri uri) {
        if (mListener != null) {
            mListener.onFragmentInteraction(uri);
        }
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        mapView = (MapView) rootView.findViewById(R.id.map);

        if (mapView!=null) {
            mapView.onCreate(null);
            mapView.onResume();
            mapView.getMapAsync(this);
        }

    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }

        mActivity = (MainActivity) context;
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    @Override
    public void onMapReady(GoogleMap googleMap) {

        progress = new ProgressDialog(getContext());
        progress.setMessage(getText(R.string.loading));
        progress.setCancelable(false); // disable dismiss by tapping outside of the dialog
        progress.show();

        MapsInitializer.initialize(getContext());
        mGoogleMap = googleMap;

        mGoogleMap.setMapType(GoogleMap.MAP_TYPE_NORMAL);

        needUpdate();
        getLocationPermission();

    }

    /**
     * Pide los permisos de acceso a la localización.
     * Setea un listener que se lanza cuando el usuario cambia de localización
     */
    public void getLocationPermission () {

        if (ContextCompat.checkSelfPermission(getActivity().getApplicationContext(),
                android.Manifest.permission.ACCESS_FINE_LOCATION)
                == PackageManager.PERMISSION_GRANTED) {

            mLocationPermissionGranted = true;
            mGoogleMap.setMyLocationEnabled(true);

            locationManager = (LocationManager) getActivity().getSystemService(Context.LOCATION_SERVICE);
            Criteria criteria = new Criteria();
            Location location = locationManager.getLastKnownLocation(locationManager.getBestProvider(criteria, false));

            if (location != null) {
                setCurrentPosition(location);
            }

            locationListener = new LocationListener() {

                public void onLocationChanged(Location location) {

                    //Setea la posición actual del usuario en el mapa
                    setCurrentPosition (location);

                    //Si se alcanzó un punto, incrementamos y pasamos a la siguiente pantalla.
                    Point p = pointReached(location);

                    if (p!=null) {

                        Intent i = new Intent(mActivity,DetailActivity.class);
                        i.putExtra("point",p);
                        mActivity.finish();
                        mActivity.startActivity (i);

                    }

                }

                public void onStatusChanged(String provider, int status, Bundle extras) {}

                public void onProviderEnabled(String provider) {}

                public void onProviderDisabled(String provider) {}

            };

            //Cada cuanto se lanza el listener
            locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 5, locationListener);

        } else {
            ActivityCompat.requestPermissions(getActivity(),
                    new String[]{android.Manifest.permission.ACCESS_FINE_LOCATION},
                    PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION);

            getLocationPermission();

        }

    }

    /**
     * Coloca los duendes sobre el mapa
     */
    private void putPointsInMap () {

        if (City.points != null) {

            for (int i = 0; i < City.points.size(); i++) {
                //Log.v("PETER_CRAB","lat/long = "+points.get(i).getLatitude()+"/"+points.get(i).getLongitude());
                Point aux = City.points.get(i);

                if (userInfo.hasVisited(i)) {
                    aux.setVisited(true);
                }

                markPoint(aux);

            }

            progress.dismiss();

        }
    }

    /**
     * Chequea si hay cambio en la información de los puntos
     * en base de datos, de ser así los vuelve a cargar de firebase.
     * En caso contrario, los carga de temporal.
     */
    public void needUpdate () {

        FirebaseDatabase database = FirebaseDatabase.getInstance();
        DatabaseReference lastUpdateRef = database.getReference("cities/Barcelona/lastUpdate");

        lastUpdateRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                SharedPreferencesUtil shared = new SharedPreferencesUtil(getActivity());

                String lastUpdate   = dataSnapshot.getValue(String.class);
                String myLastUpdate = shared.getLastUpdate();

                shared.setLastUpdate(lastUpdate);

                if (myLastUpdate== null || !lastUpdate.contentEquals(myLastUpdate)) {

                    loadPointsFromFirebase();

                }
                else {

                    if (City.points == null) {
                        new SharedPreferencesUtil(getActivity()).setPointsFromShared();
                    }

                    putPointsInMap();

                }

            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.w("PETER_CRAB", "Failed to read value.", error.toException());
            }
        });

    }

    /**
     * Método que carga los puntos de base de datos de Firebase Database.
     */
    public void loadPointsFromFirebase () {

        FirebaseDatabase database = FirebaseDatabase.getInstance();
        DatabaseReference pointsRef = database.getReference("cities/Barcelona/points");

        pointsRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                // This method is called once with the initial value and again
                // whenever data at this location is updated.

                GenericTypeIndicator<List<Point>> genericTypeIndicator = new GenericTypeIndicator<List<Point>>() {
                };

                City.points = dataSnapshot.getValue(genericTypeIndicator);

                new SharedPreferencesUtil(getActivity()).setPointsInShared();

                putPointsInMap();

            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.w("PETER_CRAB", "Failed to read value.", error.toException());
            }
        });

    }

    /**
     * Centra la camara en la posición pasada por parámetro
     * @param location
     */
    private void setCurrentPosition (Location location) {

        //Log.v("PETERCRAB",location.toString());

        mGoogleMap.animateCamera(CameraUpdateFactory.newLatLngZoom (new LatLng(location.getLatitude(), location.getLongitude() ),APPROPRIATE_ZOOM_LEVEL_MAP));


    }

    /**
     * Marca un punto con un duende.
     * @param p
     */
    public void markPoint (Point p) {

        //mMap.animateCamera(CameraUpdateFactory.newLatLngZoom (new LatLng(p.getLatitude(), p.getLongitude() ),APPROPRIATE_ZOOM_LEVEL_MAP));
        int resource = R.drawable.pointer;

        if (p.isVisited()) {
            resource = R.drawable.pointer_visited;
        }

        MarkerOptions marker = new MarkerOptions()
                .position(new LatLng(p.getLatitude(), p.getLongitude()))
                //TODO: .title(p.getTitle())
                .icon(BitmapDescriptorFactory.fromResource(resource));

        mGoogleMap.addMarker(marker);

    }

    /**
     * Comprueba si la localización pasada por parámetro está cerca de algún punto.
     * @param currentLocation
     * @return
     */
    public Point pointReached (Location currentLocation) {

        if (City.points != null) {

            for(int i=0;i<City.points.size();i++){

                Point p = City.points.get(i);

                if (p.isVisited()) continue;

                if (currentLocation.distanceTo(Point.getCurrentLocation(p.getLatitude(),p.getLongitude())) <= POINT_REACHED_DISTANCE) {
                    p.setVisited(true);
                    visitPoint (i);
                    //mGoogleMap.clear();
                    //putPointsInMap();
                    return p;
                }
            }

        }
        return null;
    }

    /**
     * Marca un punto cuyo id es pasado por parámetro.
     * Incrementa la puntuación y guarda la actualización en base de datos.
     * @param i
     */
    public void visitPoint (Integer i) {

        userInfo.incScore(POINTS_PER_ELF);
        userInfo.visitPoint(i);

        FirebaseDatabase database = FirebaseDatabase.getInstance();

        mAuth = FirebaseAuth.getInstance();

        currentUser = mAuth.getCurrentUser();

        DatabaseReference dbRef = database.getReference("users").child(currentUser.getUid());

        Map<String,Object> fields = new HashMap<String,Object>();

        fields.put("score", userInfo.getScore() );
        fields.put("order", - userInfo.getScore() );
        fields.put("visited_points", userInfo.getVisited_points());

        dbRef.updateChildren(fields);

    }

    public interface OnFragmentInteractionListener {
        void onFragmentInteraction(Uri uri);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode,
                                           @NonNull String permissions[],
                                           @NonNull int[] grantResults) {
        mLocationPermissionGranted = false;
        switch (requestCode) {
            case PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION: {
                // If request is cancelled, the result arrays are empty.
                if (grantResults.length > 0
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    mLocationPermissionGranted = true;

                }
            }
        }

    }


}
